##########################################################################
# test_task_imview.py
#
# Copyright (C) 2018
# Associated Universities, Inc. Washington DC, USA.
#
# This script is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
# License for more details.
#
# [Add the link to the JIRA ticket here once it exists]
#
# Based on the requirements listed in plone found here:
# https://casadocs.readthedocs.io/en/stable/api/tt/casaviewer.imview.html
#
#
# Because the viewer has no way to specify specific dimensions and because the size
# of the bitmaps that it produces are dependent on the dimensions of the viewer, it
# is difficult to create good tests to verify that the viewer works properly. As a
# result, this test only verifies that it produces the expected file.
#
###########################################################################
import os
import shutil
import os.path
import unittest
from casatools import coordsys, image
from casaviewer import imview

class imview_test(unittest.TestCase):
    
    @classmethod
    def setUpClass(self):
        """creating test images"""
        self.linim = "lin"+str(os.getpid())+".im"
        self.outfiles = { }
        for t in ['jpg', 'pdf', 'eps', 'ps', 'png', 'xbm', 'xpm', 'ppm']:
            self.outfiles[t] = "lin"+str(os.getpid())+"."+t
        local_cs = coordsys( )
        local_ia = image( )
        local_cs.addcoordinate(linear=2)
        local_ia.fromshape(self.linim,[20,20],csys=local_cs.torecord( ))
        xx = local_ia.getchunk( )
        xx[9:19,:] = 1
        local_ia.putchunk(xx)
        local_ia.done( )
        local_cs.done( )

    @classmethod
    def tearDownClass(self):
        """removing test images and outfiles"""
        if os.path.exists(self.linim):
            shutil.rmtree(self.linim)
        for outfileType in self.outfiles:
            thisOutfile = self.outfiles[outfileType]
            if os.path.exists(thisOutfile):
                os.system('rm -rf ' + thisOutfile)
        
    def test_xbm(self):
        """Test production of Xbm file"""
        imview(self.linim,out=self.outfiles['xbm'])
        self.assertTrue(os.path.isfile(self.outfiles['xbm']),"viewer failed to produce an Xbm file")

    def test_jpg(self):
        """Test production of JPEG file"""
        imview(self.linim,out=self.outfiles['jpg'])
        self.assertTrue(os.path.isfile(self.outfiles['jpg']),"viewer failed to produce an JPEG file")

    def test_pdf(self):
        """Test production of PDF file"""
        imview(self.linim,out=self.outfiles['pdf'])
        self.assertTrue(os.path.isfile(self.outfiles['pdf']),"viewer failed to produce an PDF file")

    def test_eps(self):
        """Test production of EPS file"""
        imview(self.linim,out=self.outfiles['eps'])
        self.assertTrue(os.path.isfile(self.outfiles['eps']),"viewer failed to produce an EPS file")

    def test_ps(self):
        """Test production of PS file"""
        imview(self.linim,out=self.outfiles['ps'])
        self.assertTrue(os.path.isfile(self.outfiles['ps']),"viewer failed to produce an PS file")

    def test_xpm(self):
        """Test production of XPM file"""
        imview(self.linim,out=self.outfiles['xpm'])
        self.assertTrue(os.path.isfile(self.outfiles['xpm']),"viewer failed to produce an XPM file")

    def test_ppm(self):
        """Test production of PPM file"""
        imview(self.linim,out=self.outfiles['ppm'])
        self.assertTrue(os.path.isfile(self.outfiles['ppm']),"viewer failed to produce an PPM file")

    def test_png(self):
        """Test production of PNG file"""
        imview(self.linim,out=self.outfiles['png'])
        self.assertTrue(os.path.isfile(self.outfiles['png']),"viewer failed to produce an PNG file")

if __name__ == '__main__':
    unittest.main()
